import java.io.* ;

/**
 *  A simple input class to read values typed at the command line.  If
 *  an error occurs during input, any exceptions thrown are caught and
 *  a default value returned.
 */
public class KeyboardInput
{
    public final synchronized int readInteger()
    {
        //
        //  The place to hold the data received from the keyboard so
        //  that we can parse it to find the int value.
        //
        String input = "" ;
        //
        //  Read a String of characters from the keyboard.
        //
        try
        {
            input = in.readLine() ;
        }
        catch (IOException e) {}
        //
        //  Parse the String to construct an int value.
        //
        int val = 0 ;
        try
        {
            val = Integer.parseInt(input) ;
        }
        catch (NumberFormatException e) {}
        return val ;
    }

    public final synchronized long readLong()
    {
        String input = "" ;
        //
        //  Read a String of characters from the keyboard.
        //
        try
        {
            input = in.readLine() ;
        }
        catch (IOException e) {}
        //
        //  Parse the String to construct a long value.
        //
        long val = 0L ;
        try
        {
            val = Long.parseLong(input) ;
        }
        catch (NumberFormatException e) {}
        return val ;
    }
    public final synchronized double readDouble()
    {
        String input = "" ;
        //
        //  Read a String of characters from the keyboard.
        //
        try
        {
            input = in.readLine() ;
        }
        catch (IOException e) {}
        //
        //  Parse the String to construct a double value.
        //
        double val = 0.0D ;
        try
        {
            val = (Double.valueOf(input)).doubleValue() ;
        }
        catch (NumberFormatException e) {}
        return val ;
    }
    public final synchronized float readFloat()
    {
        String input = "" ;
        //
        //  Read a String of characters from the keyboard.
        //
        try
        {
            input = in.readLine() ;
        }
        catch (IOException e) {}
        //
        //  Parse the String to construct a float value.
        //
        float val = 0.0F ;
        try
        {
            val = (Float.valueOf(input)).floatValue() ;
        }
        catch (NumberFormatException e) {}
        return val ;
    }
    public final synchronized char readCharacter()
    {
        //
        //  No need to parse anything, just get a character and return
        //  it..
        //
        char c = ' ' ;
        try
        {
            c = (char)in.read() ;
        }
        catch (IOException e) {}
        return c ;
    }
    public final synchronized String readString()
    {
        //
        //  No need to parse anything, just get a string and return
        //  it..
        //
        String s = "";
        try
        {
            s = in.readLine() ;
        }
        catch (IOException e) {}
        return s ;
    }
    private final BufferedReader in =
        new BufferedReader(new InputStreamReader(System.in)) ;
}
