package ScopeTest;

class ClassScope
{
    // This method illustrates that the private variables of
    // any ClassScope object can be accessed in a ClassScope
    // method.
    public void alter(final ClassScope c)
    {
        // c is the parameter, not the object the method was
        // called for, but the private variable n can still be
        // accessed.
        c.n = 123 ;
    }

    // Display the value of an instance object by using
    // the value of the private variable n.
    public String toString()
    {
        return new Integer(n).toString() ;
    }

    // A private variable only accessible with the scope of
    // this class.
    private int n = 1 ;
}

// Use this class to test the behaviour of class scope.
public class ScopeTest
{
    public static void main(String[] args)
    {
        // OK to create ClassScope objects and
        // use their public methods.
        ClassScope a = new ClassScope() ;
        ClassScope b = new ClassScope() ;

        // Note:
        // The private variable n belonging to the object
        // referenced by b can be modified by a method called
        // for an object of the same class referenced by a.
        a.alter(b) ;

        System.out.println("Object a: " + a) ;
        System.out.println("Object b: " + b) ;

        // BUT cannot break encapsulation of a ClassScope object
        // in this class.
        // Uncommenting this line will lead to a compilation error.
        // This will be true wherever it could be moved to within
        // additional methods added to this class.
        // a.n = 123 ;
    }
}
